#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

"""
This module provides the metaclass DefinitionsTracker, which is a common
subclass of
L{InterfaceClass<harold.prog.interface.interface_class.InterfaceClass>} and
L{NsAwareClass<harold.prog.namespace.nsaware_class.NsAwareClass>}.
 
@see: L{harold.prog.interface}
@see: L{harold.prog.namespace}
"""

import sre

_PRIVATE = sre.compile ('_.*__.*')

def public_direct_members (cls, include__=True, known_public=()):
    try:
        defs = cls._definitions ()
    except AttributeError:
        defs = {}
    return [
        (name,member)
        for name,member in cls.__dict__.iteritems ()
        if name in known_public
        or (
            not _PRIVATE.match (name)
            and (not name.startswith('__') or include__)
        )
        or getattr (member, "__harold_public__", False)
        or defs.has_key (name)
     ]

class DefinitionsTracker (type):
    """
    A class keeping track, in its L{_definitions} attribute, of the
    superclasses defining each of its inherited methods.
    """

    def __init__ (cls, cls_name, direct_superclasses, direct_members):
        super (DefinitionsTracker, cls).__init__ (
            cls_name, direct_superclasses, direct_members )

        __definitions = {}
        for ancestor in cls.mro()[1:][::-1]:
            public = __definitions.keys()
            for name, member in public_direct_members (ancestor,
                                                       known_public=public):
                defs = __definitions.get (name, ())
                # ancestor must be inserted in defs
                # and supercedes all its own ancestors
                defs = [ancestor] + [
                    a for a in defs if not issubclass (ancestor, a)
                ]
                __definitions[name] = defs

        cls.__definitions = __definitions
            

    def _definitions (self, name=None):
        """
        Return a tuple of the superclasses defining the named member.

        If C{name} is None or not given, a dict of the lists is returned.

        The following points are important:

          - of two related superclass (i.e. one is a subclass of the other),
            only the most specific (i.e. the subclass) is in C{_definitions}.

          - the order of the tuple is the same order as this classe's C{mro}.

        """
        if name is None:
            return dict (self.__definitions)
        else:
            return tuple (self.__definitions.get (name, ()))
