#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

"""
@group Warnings: AbstractClassInstantiationWarning AbstractOverridesConcereteWarning
"""

# ==== Warnings (not blocking by default) ====

class AbstractClassInstantiationWarning (Warning):
    """
    Issued whenever an abstract class without any default implementation is
    instantiated.
    """
    pass

class AbstractOverridesConcreteWarning (Warning):
    """
    Issued whenever an abstract member overrides a concrete one, which is
    usually the consequence of an inappropriate order in the superclasses.

    Remember to put concrete classes I{before} abstract ones in the
    superclasses list.
    """
    pass

# ==== Exceptions (happen generally at runtime ) ====

class AbstractMemberAccessException (Exception):
    """
    Raised whenever an abstract member is accessed.

    No direct instance of this class is ever raised; instances of its
    subclasses are raised instead.
    """
    pass

class AbstractAttributeException (AbstractMemberAccessException):
    """
    Raised whenever an abstract attribute is accessed (get or set).
    """
    pass

class AbstractMethodInvocationException (AbstractMemberAccessException):
    """
    Raised whenever an abstract method is invoked.
    """
    pass

class ImplementationException (Exception):
    """
    Raised whenever an abstract class is assigned an inconsistent default
    implementation.
    """
    pass

