#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

"""
I define classes for declaring abstract members.
Note that those classes can be imported directly from L{harold.prog.abstract}.

@summary:
"""

from harold.prog.abstract.exceptions import (
    AbstractAttributeException,
    AbstractMethodInvocationException,
)

from harold.callabletools import CallableWrapper
from types                import MethodType

class _AbstractMember (object):
    def __init__ (self, f):
        self.__f = f

    @property
    def __doc__ (self):
        return self.__f.__doc__

def _attribute_raise (*args, **kw):
    raise AbstractAttributeException

class _AbstractAttribute (_AbstractMember, property):
    def __init__ (self, f, *args):
        property.__init__ (self, *args)
        _AbstractMember.__init__ (self, f)


class attr_r (_AbstractAttribute):
    """
    Abstract read-only attribute.

    See L{harold.prog.abstract} for more detail.
    """
    def __init__ (self, f):
        _AbstractAttribute.__init__ (self, f, _attribute_raise)

class attr_rw (_AbstractAttribute):
    """
    Abstract read-write attribute.

    See L{harold.prog.abstract} for more detail.
    """
    def __init__ (self, f):
        _AbstractAttribute.__init__ (self, f, _attribute_raise,
                                              _attribute_raise)


class method (_AbstractMember, CallableWrapper):
    """
    An abstract method.
    """
    def __init__ (self, f):
        _AbstractMember.__init__ (self, f)
        CallableWrapper.__init__ (self, f)
    def __call__ (self, *args, **kw):
        raise AbstractMethodInvocationException
