#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

"""
@group Warnings: AmbiguousMemberWarning AmbiguousMembersDetected
"""
class AmbiguousMemberException (Exception):
    """
    Raised whenever an explicitly ambiguous member is accessed in a context
    where no namespace has been set for it.
    """
    pass

class AmbiguousMemberWarning (Warning):
    """
    Issued whenever an implicitly (i.e. automatically detected) ambiguous
    member is accessed in a context where no namespace has been set for it.
    """
    pass

class AmbiguousClassWarning (Warning):
    """
    Issued whenever a L{harold.prog.namespace.nsaware_class.NsAwareClass}
    declares ambiguous members implicitely.

    It is recommended to make these declarations explicit rather than to filter
    this warning out.
    """
    pass

class InvalidContextWarning (Warning):
    """
    Issued by AmbiguousMember whenever an attempt is made to assign a namespace
    to a part of the interactive module (<stdin> or <console>).
    """
    pass

class ConflictingNamespacesException (Exception):
    """
    Raised by AmbiguousMember whenever a context already having an assigned
    namespace is assigned another context
    """
    pass

class NoDefinitionInNamespaceException (Exception):
    """
    Raised by AmbiguousMember whenever an assigned namespace does not
    define this ambiguous member.
    """
    pass

class UnrecognizedContextException (Exception):
    """
    Raised by AmbiguousMember whenever an abject used as a context is not
    recognize.
    """
    pass
