#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

from harold.prog._deftracker import DefinitionsTracker
from harold.prog.namespace import ambiguous
from harold.prog.namespace.exceptions import AmbiguousClassWarning

from warnings import warn

class NsAwareClass (DefinitionsTracker):

    # this variable is used to make the warning messages more explicit
    # it works fine whith 0 when NsAwareClass is used directly,
    # but should be overridden in its subclasses
    __warning_level = 0

    def __init__ (cls, cls_name, direct_superclasses, direct_members):
        super (NsAwareClass, cls).__init__ (
            cls_name, direct_superclasses, direct_members )

        override = direct_members.get ("__override__", {})

        detected = []
        for name, defs in cls._definitions().iteritems():
            if len (defs) > 1 and name not in direct_members:
                # member is not implemented in this class;
                # is the first definition referenced in __override__ ?
                ignore = override.get (defs[0], ())
                if isinstance (ignore, type):
                    ignore = (ignore,)
                if len (ignore) == 0 \
                or [ d for d in defs[1:] if d not in ignore ]:
                    detected.append (name)

        if detected:
            ambiguous (cls=cls, warn_only=True, automate=True, *detected)
            warn (
                " ".join (detected),
                AmbiguousClassWarning,
                2 + cls.__warning_level,
            )
