#!/usr/bin/python2.4

#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

import unittest

from harold.callabletools import *

class TestUnbounded (unittest.TestCase):
    def testRightComparison (self):
        self.failUnless (1 <  UNBOUNDED)
        self.failUnless (1 <= UNBOUNDED)
        self.failIf     (1 == UNBOUNDED)
        self.failIf     (1 >= UNBOUNDED)
        self.failIf     (1 >  UNBOUNDED)

    def testLeftComparison (self):
        self.failIf     (UNBOUNDED <  1)
        self.failIf     (UNBOUNDED <= 1)
        self.failIf     (UNBOUNDED == 1)
        self.failUnless (UNBOUNDED >= 1)
        self.failUnless (UNBOUNDED >  1)

    def testSelfComparison (self):
        self.failIf     (UNBOUNDED <  UNBOUNDED)
        self.failUnless (UNBOUNDED <= UNBOUNDED)
        self.failUnless (UNBOUNDED == UNBOUNDED)
        self.failUnless (UNBOUNDED >= UNBOUNDED)
        self.failIf     (UNBOUNDED >  UNBOUNDED)

    def testLeftOperations (self):
        self.assertEquals (UNBOUNDED + 1, UNBOUNDED)
        self.assertEquals (UNBOUNDED - 1, UNBOUNDED)
        self.assertEquals (UNBOUNDED * 2, UNBOUNDED)
        self.assertEquals (UNBOUNDED / 2, UNBOUNDED)
        self.assertEquals (UNBOUNDED % 2, UNBOUNDED)

    def testRightOperations (self):
        self.assertEquals (1 + UNBOUNDED + 1, UNBOUNDED)
        self.assertEquals (2 * UNBOUNDED * 2, UNBOUNDED)

class _TestBase (object):
    """
    This class is intented to be subclassed, and its subclasses are assumed to
    verify the following points:
      - they also subclass unittest.TestCase
      - they have a __call__ method
      - they have min,max,kw attributes describing the signature of __call__
        (not counting parameter self)

    This class provides all the test* methods to check that callabletools works
    fine with functions, methods, classes and instances with a __call__ method.
    """
    def setUp (self):
        f = self.callable.im_func
        self.func = f
        self.wrap = CallableWrapper (f)
        class C (object):
            __init__ = f
        self.cls = C
        class C (object):
            __call__ = f
        self.inst = C()
        if self.max >= 1:
            class desc (object):
                __get__ = MethodType
                __call__ = f
            class C (object):
                __call__ = desc()
            self.tricky = C()


    def __sign (self, c, mod=0):
        self.assertEquals (
            signature (c),
            (max (0, self.min+mod), self.max+mod, self.kw),
        )
    def __min (self, c, mod=0):
        self.assertEquals (min_args (c), max (0, self.min+mod))
    def __max (self, c, mod=0):
        self.assertEquals (max_args (c), self.max+mod)
    def __kw  (self, c):
        self.assertEquals (kw_args (c) != 0, self.kw != False)

    def testSignatureFunc (self): self.__sign (self.func, 1)
    def testMinFunc (self): self.__min (self.func, 1)
    def testMaxFunc (self): self.__max (self.func, 1)
    def testKwFunc (self): self.__kw (self.func)
    def testSignatureMethod (self): self.__sign (self.callable)
    def testMinMethod (self): self.__min (self.callable)
    def testMaxMethod (self): self.__max (self.callable)
    def testKwMethod (self): self.__kw (self.callable)
    def testSignatureWrap (self): self.__sign (self.wrap, 1)
    def testMinWrap (self): self.__min (self.wrap, 1)
    def testMaxWrap (self): self.__max (self.wrap, 1)
    def testKwWrap (self): self.__kw (self.wrap)
    def testSignatureClass (self): self.__sign (self.cls)
    def testMinClass (self): self.__min (self.cls)
    def testMaxClass (self): self.__max (self.cls)
    def testKwClass (self): self.__kw (self.cls)
    def testSignatureCall (self): self.__sign (self.inst)
    def testMinCall (self): self.__min (self.inst)
    def testMaxCall (self): self.__max (self.inst)
    def testKwCall (self): self.__kw (self.inst)
    def testSignatureTricky (self):
        if self.max: self.__sign (self.tricky, -1)
    def testMinTricky (self):
        if self.max: self.__min (self.tricky, -1)
    def testMaxTricky (self):
        if self.max: self.__max (self.tricky, -1)
    def testKwTricky (self):
        if self.max: self.__kw (self.tricky)
  
    pass

class Test_0 (_TestBase, unittest.TestCase):
    def callable (self): pass
    min,max,kw = 0,0,False
class Test_0p (_TestBase, unittest.TestCase):
    def callable (self, *args): pass
    min,max,kw = 0,UNBOUNDED,False
class Test_0pk (_TestBase, unittest.TestCase):
    def callable (self, *args, **kw): pass
    min,max,kw = 0,UNBOUNDED,True
class Test_0k (_TestBase, unittest.TestCase):
    def callable (self, **kw): pass
    min,max,kw = 0,0,True

class Test_01 (_TestBase, unittest.TestCase):
    def callable (self, x=0): pass
    min,max,kw = 0,1,False
class Test_01p (_TestBase, unittest.TestCase):
    def callable (self, x=0, *args): pass
    min,max,kw = 0,UNBOUNDED,False
class Test_01pk (_TestBase, unittest.TestCase):
    def callable (self, x=0, *args, **kw): pass
    min,max,kw = 0,UNBOUNDED,True
class Test_01k (_TestBase, unittest.TestCase):
    def callable (self, x=0, **kw): pass
    min,max,kw = 0,1,True

class Test_1 (_TestBase, unittest.TestCase):
    def callable (self, x): pass
    min,max,kw = 1,1,False
class Test_1p (_TestBase, unittest.TestCase):
    def callable (self, x, *args): pass
    min,max,kw = 1,UNBOUNDED,False
class Test_1pk (_TestBase, unittest.TestCase):
    def callable (self, x, *args, **kw): pass
    min,max,kw = 1,UNBOUNDED,True
class Test_1k (_TestBase, unittest.TestCase):
    def callable (self, x, **kw): pass
    min,max,kw = 1,1,True

class Test_12 (_TestBase, unittest.TestCase):
    def callable (self, x, y=0): pass
    min,max,kw = 1,2,False
class Test_12p (_TestBase, unittest.TestCase):
    def callable (self, x, y=0, *args): pass
    min,max,kw = 1,UNBOUNDED,False
class Test_12pk (_TestBase, unittest.TestCase):
    def callable (self, x, y=0, *args, **kw): pass
    min,max,kw = 1,UNBOUNDED,True
class Test_12k (_TestBase, unittest.TestCase):
    def callable (self, x, y=0, **kw): pass
    min,max,kw = 1,2,True

class Test_2 (_TestBase, unittest.TestCase):
    def callable (self, x, y): pass
    min,max,kw = 2,2,False
class Test_2p (_TestBase, unittest.TestCase):
    def callable (self, x, y, *args): pass
    min,max,kw = 2,UNBOUNDED,False
class Test_2pk (_TestBase, unittest.TestCase):
    def callable (self, x, y, *args, **kw): pass
    min,max,kw = 2,UNBOUNDED,True
class Test_2k (_TestBase, unittest.TestCase):
    def callable (self, x, y, **kw): pass
    min,max,kw = 2,2,True

class Test_02 (_TestBase, unittest.TestCase):
    def callable (self, x=0, y=0): pass
    min,max,kw = 0,2,False
class Test_02p (_TestBase, unittest.TestCase):
    def callable (self, x=0, y=0, *args): pass
    min,max,kw = 0,UNBOUNDED,False
class Test_02pk (_TestBase, unittest.TestCase):
    def callable (self, x=0, y=0, *args, **kw): pass
    min,max,kw = 0,UNBOUNDED,True
class Test_02k (_TestBase, unittest.TestCase):
    def callable (self, x=0, y=0, **kw): pass
    min,max,kw = 0,2,True


class TestWrapper (unittest.TestCase):
    def setUp (self):
        def f(x,y):
            "f's doc"
            pass
        self.f = f
        self.wf = CallableWrapper (f)

        class C(object):
            "C's doc"
            __call__ = f
        self.c = C
        self.wc = CallableWrapper (C)
        k = C()
        self.k = k
        self.wk = CallableWrapper (k)

    def testFName     (s): s.assertEquals (s.f.__name__,   s.wf.__name__)
    def testFDoc      (s): s.assertEquals (s.f.__doc__,    s.wf.__doc__)
    def testFModule   (s): s.assertEquals (s.f.__module__, s.wf.__module__)
    def testFFuncName (s): s.assertEquals (s.f.func_name,  s.wf.func_name)
    def testFFuncDoc  (s): s.assertEquals (s.f.func_doc,   s.wf.func_doc)
    def testCName     (s): s.assertEquals (s.c.__name__,   s.wc.__name__)
    def testCDoc      (s): s.assertEquals (s.c.__doc__,    s.wc.__doc__)
    def testCModule   (s): s.assertEquals (s.c.__module__, s.wc.__module__)
    def testKDoc      (s): s.assertEquals (s.k.__doc__,    s.wk.__doc__)
    def testKModule   (s): s.assertEquals (s.k.__module__, s.wk.__module__)

    def testGet (self):
        class C (object):
            @CallableWrapper
            def f(self, x, y):
                return x,y
        c = C()
        self.assertEquals (C.f(c,1,2), (1,2))
        self.assertEquals (c.f(1,2), (1,2))

if __name__ == "__main__":
    unittest.main ()
