#!/usr/bin/python2.4

#
# (c)2005 LIRIS - University Claude Bernard Lyon 1
# http://liris.cnrs.fr/
#
# Author: Pierre-Antoine CHAMPIN
# http://champin.net/
#
# This software is distributed under the terms of the GNU LGPL v2.1.
# See LICENSE.txt for more details.
#

import               harold
import               unittest
from warnings import filterwarnings, resetwarnings

from harold.prog._deftracker              import DefinitionsTracker
from harold.prog.namespace.nsaware_class  import NsAwareClass
from harold.prog.namespace.exceptions     import AmbiguousClassWarning

metaclass_to_test = DefinitionsTracker

if not harold.lightweight:
    class TestMcDefinitionsTracker (unittest.TestCase):
        def setUp (self):
            filterwarnings ("ignore", category=AmbiguousClassWarning)
            class A (object):
                __metaclass__ = metaclass_to_test
                def a (self): pass
                def ab (self): pass
                def abc (self): pass
                def abcd (self): pass
                def abcde (self): pass
                def abce (self): pass
                def abd (self): pass
                def abde (self): pass
                def abe (self): pass
                def ac (self): pass
                def acd (self): pass
                def acde (self): pass
                def ace (self): pass
                def ad (self): pass
                def ade (self): pass
                def ae (self): pass

            class B (A):
                def ab (self): pass
                def abc (self): pass
                def abcd (self): pass
                def abcde (self): pass
                def abce (self): pass
                def abd (self): pass
                def abde (self): pass
                def abe (self): pass

                def b (self): pass
                def bc (self): pass
                def bcd (self): pass
                def bcde (self): pass
                def bce (self): pass
                def bd (self): pass
                def bde (self): pass
                def be (self): pass

            class C (A):
                __metaclass__ = metaclass_to_test
                def abc (self): pass
                def abcd (self): pass
                def abcde (self): pass
                def abce (self): pass
                def ac (self): pass
                def acd (self): pass
                def acde (self): pass
                def ace (self): pass
                def bc (self): pass
                def bcd (self): pass
                def bcde (self): pass
                def bce (self): pass

                def c (self): pass
                def cd (self): pass
                def cde (self): pass
                def ce (self): pass

            class D (B,C):
                __metaclass__ = metaclass_to_test
                def abcd (self): pass
                def abcde (self): pass
                def abd (self): pass
                def abde (self): pass
                def acd (self): pass
                def acde (self): pass
                def ad (self): pass
                def ade (self): pass
                def bcd (self): pass
                def bcde (self): pass
                def bd (self): pass
                def bde (self): pass
                def cd (self): pass
                def cde (self): pass

                def d (self): pass
                def de (self): pass

            class E (D):
                __metaclass__ = metaclass_to_test
                def abcde (self): pass
                def abce (self): pass
                def abde (self): pass
                def abe (self): pass
                def acde (self): pass
                def ace (self): pass
                def ade (self): pass
                def ae (self): pass
                def bcde (self): pass
                def bce (self): pass
                def bde (self): pass
                def be (self): pass
                def cde (self): pass
                def ce (self): pass
                def de (self): pass

                def e (self): pass

            self.A = A
            self.B = B
            self.C = C
            self.D = D
            self.E = E

            resetwarnings()

        def test_a (self):
            name = 'a'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.A,))
            self.assertEquals (self.E._definitions (name), (self.A,))
        def test_ab (self):
            name = 'ab'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.B,))
        def test_abc (self):
            name = 'abc'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            if issubclass (metaclass_to_test, NsAwareClass):
                self.assertEquals (self.E._definitions (name), (self.D,))
            else:
                self.assertEquals (self.E._definitions (name), (self.B, self.C))
        def test_abcd (self):
            name = 'abcd'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_abcde (self):
            name = 'abcde'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_abce (self):
            name = 'abce'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            if issubclass (metaclass_to_test, NsAwareClass):
                self.assertEquals (self.E._definitions (name), (self.D,))
            else:
                self.assertEquals (self.E._definitions (name), (self.B, self.C))
        def test_abde (self):
            name = 'abde'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_abe (self):
            name = 'abe'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.B,))
        def test_ac (self):
            name = 'ac'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.C,))
        def test_acd (self):
            name = 'acd'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_acde (self):
            name = 'acde'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_ace (self):
            name = 'ace'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.C,))
        def test_ad (self):
            name = 'ad'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.A,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_ade (self):
            name = 'ade'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.A,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_ae (self):
            name = 'ae'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), (self.A,))
            self.assertEquals (self.C._definitions (name), (self.A,))
            self.assertEquals (self.D._definitions (name), (self.A,))
            self.assertEquals (self.E._definitions (name), (self.A,))
        def test_b (self):
            name = 'b'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.B,))
        def test_bc (self):
            name = 'bc'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            if issubclass (metaclass_to_test, NsAwareClass):
                self.assertEquals (self.E._definitions (name), (self.D,))
            else:
                self.assertEquals (self.E._definitions (name), (self.B, self.C))
        def test_bcd (self):
            name = 'bcd'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_bcde (self):
            name = 'bcde'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_bce (self):
            name = 'bce'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B, self.C))
            if issubclass (metaclass_to_test, NsAwareClass):
                self.assertEquals (self.E._definitions (name), (self.D,))
            else:
                self.assertEquals (self.E._definitions (name), (self.B, self.C))
        def test_bd (self):
            name = 'bd'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_bde (self):
            name = 'bde'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_be (self):
            name = 'be'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.B,))
            self.assertEquals (self.E._definitions (name), (self.B,))
        def test_c (self):
            name = 'c'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.C,))
        def test_cd (self):
            name = 'cd'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_cde (self):
            name = 'cde'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_ce (self):
            name = 'ce'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), (self.C,))
            self.assertEquals (self.E._definitions (name), (self.C,))
        def test_d (self):
            name = 'd'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), ())
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_de (self):
            name = 'de'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), ())
            self.assertEquals (self.E._definitions (name), (self.D,))
        def test_e (self):
            name = 'e'
            self.assertEquals (self.A._definitions (name), ())
            self.assertEquals (self.B._definitions (name), ())
            self.assertEquals (self.C._definitions (name), ())
            self.assertEquals (self.D._definitions (name), ())
            self.assertEquals (self.E._definitions (name), ())

if __name__ == "__main__":
    unittest.main ()
    
